import pygame
import math
import data
import os

class Map:
    def __init__(self,map_file_name,game,level):
        self.game = game
        self.tile_width = 75
        self.tile_height = 75
        self.columns = 0
        self.rows = 0
        self.width = 0
        self.height = 0
        self.map = []
        self.wall_rects = []
        
        # These are assigned based on level and are sprites or background images
        self.bg = None
        self.fg = None
        self.wall = None 
        
        # load map from csv file and
        # populate self.map, self.columns and self.rows
        self.loadmap(data.filepath(map_file_name))
        
        # assign resources for images
        # background, bgtile, wall
        if level==0:
            self.bg = self.game.resource['menu_bg']
            self.fg = None
            self.wall = None
        if level==1:
            self.bg = None #self.game.resource['water1']
            self.fg = self.game.resource['fg1']
            self.wall = self.game.resource['Forest-1']
        elif level==2:
            self.bg = None
            self.fg = self.game.resource['Forest-2']
            self.wall = self.game.resource['Forest-1']
        elif level==3:
            self.bg = None
            self.fg = self.game.resource['ice']
            self.wall = self.game.resource['lava']
        elif level==4:
            self.bg = self.game.resource['bg4']
            self.fg = None
            self.wall = self.game.resource['asteroid']
        
        self.tiled = pygame.Surface((self.columns*self.tile_width,self.rows*self.tile_height)).convert()

        # load rects for collision into self.wall_rects
        for y in xrange(self.rows):
            for x in xrange(self.columns):
                if self.map[y][x].is_wall():
                    w = x*self.tile_width
                    h = y*self.tile_height
                    temp = pygame.Rect(w,h,self.tile_width,self.tile_height)
                    self.wall_rects.append(temp)


    def loadmap(self,map_file):
        f = open(map_file,'rb')

        # how many columns in file
        self.columns = 0
        self.rows = 0
        for line in f:
            self.rows = self.rows + 1
            temprow = line.split(',')
            temp = []
            if self.columns < 1:
                self.columns = len(temprow)
            for x in range(self.columns):
                temp.append(Tile(int(temprow[x])))
            self.map.append(temp)

        f.close()
        self.width = self.columns * self.tile_width
        self.height = self.rows * self.tile_height
       

    def get_viewable(self,x,y,width,height):
        xtile = x/self.tile_width  # tile number of upper-left corner
        ytile = y/self.tile_height
        xtileoffset = x%self.tile_width # tile offset of upper-left corner
        ytileoffset = y%self.tile_height
        tiles_x = width / self.tile_width
        tiles_y = height / self.tile_height
        self.viewable = pygame.Surface((width,height))
        graphRect = pygame.Rect(0,0,self.tile_width,self.tile_height)
        
        if self.bg:
            self.viewable.blit(self.bg,(0,0))
        
        for y in xrange(tiles_y+1):
            for x in xrange(tiles_x+2):
                xtpos = x + xtile
                ytpos = y + ytile
                if xtpos >= self.columns:
                    xtpos = xtpos - self.columns
                if ytpos >= self.rows:
                    ytpos = ytpos - self.rows
                if x==0 and y > 0:
                    graphRect = graphRect.move([-(tiles_x-1) * self.tile_width,self.tile_height])
                if x>0:
                    graphRect = graphRect.move([self.tile_width,0])
                if self.map[ytpos][xtpos].is_wall():
                    self.viewable.blit(self.wall,(x*self.tile_width-xtileoffset,y*self.tile_width-ytileoffset))
                else:
                    if self.fg:
                        self.viewable.blit(self.fg,(x*self.tile_width-xtileoffset,y*self.tile_width-ytileoffset))
        
        return self.viewable
       

    def collision(self,rect):
        # Determine if there has been a collision between passed-in rect and any walls
        if rect.collidelist(self.wall_rects)>-1:
            return True
        offsety = -self.height if rect.bottom > self.height else 0
        offsetx = -self.width if rect.right > self.width else 0
        rect.move_ip(offsetx,0)
        if rect.collidelist(self.wall_rects)>-1:
            return True
        rect.move_ip(0,offsety)
        if rect.collidelist(self.wall_rects)>-1:
            return True
        rect.move_ip(-offsetx,0)
        if rect.collidelist(self.wall_rects)>-1:
            return True            
        return False


    def find_distance(self,x1,y1,x2,y2):
        return math.sqrt(math.pow(x2-x1,2)+math.pow(y2-y1,2))


    def find_direction(self,x1,y1,x2,y2):
        # from point A to point B
        return x2-x1,y2-y1


    def find_fastest_direct_path(self,pointA,pointB):
        # from point A to point B
        x1,y1 = pointA
        x2,y2 = pointB
        
        # detects map wraparound
        ymod = self.height
        xmod = self.width
        
        # Determine fastest way to get to a point through looping
        dx = x2-x1,x2-x1+xmod,x2-x1-xmod
        dy = y2-y1,y2-y1+ymod,y2-y1-ymod
        dxa = math.fabs(x2-x1),math.fabs(x2-x1+xmod),math.fabs(x2-x1-xmod)
        dya = math.fabs(y2-y1),math.fabs(y2-y1+ymod),math.fabs(y2-y1-ymod)
      
        temp_dist = 100000000
        for _y in dy:
            for _x in dx:
                temp_dist_t = self.find_distance(0,0,_x,_y)
                if temp_dist_t < temp_dist:
                    temp_x = _x
                    temp_y = _y
                    temp_dist = temp_dist_t
        return temp_x,temp_y


    def find_real_path(self,pointA,pointB):
        pass
        

class Tile:

    # tile type flags
    WALL = 0x1 #not passable
    
    def __init__(self, _type=0):
        self.type = _type

    def is_wall(self):
        return Tile.WALL & self.type

    def set_flag(self,flag):
        self.type = self.type & flag

    def clear_flag(self,flag):
        self.type = self.type & ~flag



